/*
   vdev: a virtual device manager for *nix
   Copyright (C) 2014  Jude Nelson

   This program is dual-licensed: you can redistribute it and/or modify
   it under the terms of the GNU General Public License version 3 or later as 
   published by the Free Software Foundation. For the terms of this 
   license, see LICENSE.GPLv3+ or <http://www.gnu.org/licenses/>.

   You are free to use this program under the terms of the GNU General
   Public License, but WITHOUT ANY WARRANTY; without even the implied 
   warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
   See the GNU General Public License for more details.

   Alternatively, you are free to use this program under the terms of the 
   Internet Software Consortium License, but WITHOUT ANY WARRANTY; without
   even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
   For the terms of this license, see LICENSE.ISC or 
   <http://www.isc.org/downloads/software-support-policy/isc-license/>.
*/

#ifndef _VDEV_UTIL_H_
#define _VDEV_UTIL_H_

#include <stdio.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdbool.h>
#include <errno.h>
#include <unistd.h>
#include <dirent.h>
#include <libgen.h>
#include <signal.h>
#include <sys/stat.h>
   
#define VDEV_CALLOC(type, count) (type*)calloc( sizeof(type) * (count), 1 )

#define vdev_strdup_or_null( str )  (str) != NULL ? strdup(str) : NULL

typedef int (*vdev_dirent_loader_t)( char const*, void* );

// I/O functions 
ssize_t modprobe_read_uninterrupted( int fd, char* buf, size_t len );
int modprobe_read_file( char const* path, char* buf, size_t len );

// directory I/O
int vdev_load_all( char const* dir_path, vdev_dirent_loader_t loader, void* cls );

// misc
char* strncpy_t(char* out, size_t outsz, const char* in, size_t insz);
FILE *epopen(const char *cmd,  pid_t *pid);
char* fullpath( char const* root, char const* path, char* dest );

void daemonize();

int sysfs_get_parent_device(char const* dev_path, char** ret_parent_device, size_t* ret_parent_device_len);
bool sysfs_get_parent_with_subsystem(char const* device_path, char const* subsystem_name);
#endif  // _VDEV_UTIL_H_
